package dapp

import (
	"encoding/json"
	"fmt"

	"gitlab.33.cn/link33/chain33-sdk-go/client"
	. "gitlab.33.cn/link33/chain33-sdk-go/dapp/broker"
	"gitlab.33.cn/link33/chain33-sdk-go/dapp/storage"
	"gitlab.33.cn/link33/chain33-sdk-go/types"

	. "github.com/bitly/go-simplejson"
)

func QueryStorageByKey(prefix, url, key string) (*types.Storage, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryStorage{TxHash: key})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + storage.StorageX,
		FuncName: storage.FuncNameQueryStorage,
		Payload:  jsonraw,
	}
	// var storage types.Storage
	data, err := jsonClient.CallBack("Chain33.Query", query, ParseStorage)
	if err != nil {
		return nil, err
	}
	return data.(*types.Storage), nil
}

// 回调解析函数
func ParseStorage(raw json.RawMessage) (interface{}, error) {
	js, err := NewJson(raw)
	if err != nil {
		return nil, err
	}
	if contentStorge, ok := js.CheckGet("contentStorage"); ok {
		contentHex, _ := contentStorge.Get("content").String()
		content, _ := types.FromHex(contentHex)
		key, _ := contentStorge.Get("key").String()
		value, _ := contentStorge.Get("value").String()
		op, _ := contentStorge.Get("op").Int()
		storage := &types.Storage{Ty: storage.TyContentStorageAction, Value: &types.Storage_ContentStorage{ContentStorage: &types.ContentOnlyNotaryStorage{
			Content:              content,
			Key:                  key,
			Op:                   int32(op),
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	if linkStorge, ok := js.CheckGet("linkStorage"); ok {
		linkHex, _ := linkStorge.Get("link").String()
		link, _ := types.FromHex(linkHex)
		key, _ := linkStorge.Get("key").String()
		value, _ := linkStorge.Get("value").String()
		storage := &types.Storage{Ty: storage.TyLinkStorageAction, Value: &types.Storage_LinkStorage{LinkStorage: &types.LinkNotaryStorage{
			Link:                 link,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	if hashStorge, ok := js.CheckGet("hashStorage"); ok {
		hashHex, _ := hashStorge.Get("hash").String()
		hash, _ := types.FromHex(hashHex)
		key, _ := hashStorge.Get("key").String()
		value, _ := hashStorge.Get("value").String()
		storage := &types.Storage{Ty: storage.TyHashStorageAction, Value: &types.Storage_HashStorage{HashStorage: &types.HashOnlyNotaryStorage{
			Hash:                 hash,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}

	if encryptStorage, ok := js.CheckGet("encryptStorage"); ok {
		contentHashHex, _ := encryptStorage.Get("contentHash").String()
		contentHash, _ := types.FromHex(contentHashHex)
		encryptContentHex, _ := encryptStorage.Get("encryptContent").String()
		encryptContent, _ := types.FromHex(encryptContentHex)
		nonceHex, _ := encryptStorage.Get("nonce").String()
		nonce, _ := types.FromHex(nonceHex)
		key, _ := encryptStorage.Get("key").String()
		value, _ := encryptStorage.Get("value").String()
		storage := &types.Storage{Ty: storage.TyEncryptStorageAction, Value: &types.Storage_EncryptStorage{EncryptStorage: &types.EncryptNotaryStorage{
			EncryptContent:       encryptContent,
			ContentHash:          contentHash,
			Nonce:                nonce,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	if encryptStorage, ok := js.CheckGet("encryptShareStorage"); ok {
		contentHashHex, _ := encryptStorage.Get("contentHash").String()
		contentHash, _ := types.FromHex(contentHashHex)
		encryptContentHex, _ := encryptStorage.Get("encryptContent").String()
		encryptContent, _ := types.FromHex(encryptContentHex)
		pubKeyHex, _ := encryptStorage.Get("pubKey").String()
		pubKey, _ := types.FromHex(pubKeyHex)
		key, _ := encryptStorage.Get("key").String()
		value, _ := encryptStorage.Get("value").String()
		storage := &types.Storage{Ty: storage.TyEncryptShareStorageAction, Value: &types.Storage_EncryptShareStorage{EncryptShareStorage: &types.EncryptShareNotaryStorage{
			EncryptContent:       encryptContent,
			ContentHash:          contentHash,
			PubKey:               pubKey,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	return nil, fmt.Errorf("unknow type!")
}

func PollingEvent(prefix, url string, meta *types.Meta) (*types.InterChainEventList, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.PollingEvent{Meta: meta})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNamePollingEvent,
		Payload:  jsonraw,
	}
	var eventList types.InterChainEventList
	err = jsonClient.Call("Chain33.Query", query, &eventList)
	if err != nil {
		return nil, err
	}
	return &eventList, nil
}

func QueryInnerMeta(prefix, url string) (*types.Meta, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryInnerMeta{})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNameQueryInnerMeta,
		Payload:  jsonraw,
	}
	var meta types.Meta
	err = jsonClient.Call("Chain33.Query", query, &meta)
	if err != nil {
		return nil, err
	}
	return &meta, nil
}

func QueryOutterMeta(prefix, url string) (*types.Meta, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryOutterMeta{})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNameQueryOutterMeta,
		Payload:  jsonraw,
	}
	var meta types.Meta
	err = jsonClient.Call("Chain33.Query", query, &meta)
	if err != nil {
		return nil, err
	}
	return &meta, nil
}

func QueryCallBackMeta(prefix, url string) (*types.Meta, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryNilParam{})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNameQueryCallBackMeta,
		Payload:  jsonraw,
	}
	var meta types.Meta
	err = jsonClient.Call("Chain33.Query", query, &meta)
	if err != nil {
		return nil, err
	}
	return &meta, nil
}

func QueryOutMessage(prefix, url, inServicePair string, index uint64) (*types.Response, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryOutMessage{InServicePair: inServicePair, SequenceNum: index})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNameQueryOutMessage,
		Payload:  jsonraw,
	}
	var resp types.Response
	err = jsonClient.Call("Chain33.Query", query, &resp)
	if err != nil {
		return nil, err
	}
	return &resp, nil
}

func QueryInMessage(prefix, url, inServicePair string, index uint64) (*types.Response, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryInMessage{InServicePair: inServicePair, SequenceNum: index})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNameQueryInMessage,
		Payload:  jsonraw,
	}
	var resp types.Response
	err = jsonClient.Call("Chain33.Query", query, &resp)
	if err != nil {
		return nil, err
	}
	return &resp, nil
}

func QueryBrokerInfo(prefix, url string) (*types.BrokerInfo, error) {
	jsonClient, err := client.NewJSONClient(prefix, url)
	if err != nil {
		return nil, err
	}
	jsonraw, err := json.Marshal(&types.QueryNilParam{})
	if err != nil {
		return nil, err
	}
	query := client.Query4Jrpc{
		Execer:   prefix + BrokerX,
		FuncName: FuncNameQueryBrokerInfo,
		Payload:  jsonraw,
	}
	var resp types.BrokerInfo
	err = jsonClient.Call("Chain33.Query", query, &resp)
	if err != nil {
		return nil, err
	}
	return &resp, nil
}
